//
//  QHVCTypes.h
//  QHVCCommonKit
//
//  Created by niezhiqiang on 2019/7/17.
//  Copyright © 2019 qihoo.QHVCCommonKit. All rights reserved.
//

#ifndef QHVCTypes_h
#define QHVCTypes_h

#include <stddef.h>

#if (defined(WIN32) || defined(WIN64))
#include <WinSock2.h>
#else
#include <sys/socket.h>
#endif

#ifdef __cplusplus
extern "C" {
#endif
    
#if (defined(WIN32) || defined(WIN64))
#define EXPORT_API __declspec(dllexport)
#else
#define EXPORT_API __attribute__ ((visibility("default")))
#endif
    
typedef int CORE_HANDLE;

/// 行为
enum QHVCEType
{
    QHVCETYPE_UNKNOWN = -1, ///< 无效值 内部使用
    QHVCETYPE_PLAY = 0, ///< 观看
    QHVCETYPE_PUBLISH = 1, ///< 推流
    QHVCETYPE_PRE = 9 ///< 预调度内部使用
};

/// 协议
enum QHVCEProto
{
    QHVCEPROTO_UNKNOWN = 0, /**< 无效协议 */
    QHVCEPROTO_AUTO = 1, /**< 自动 */
    QHVCEPROTO_RELAY = 2, /**< Relay */
    QHVCEPROTO_RTMP = 3, /**< RTMP */
    QHVCEPROTO_RTMP_1 = 4, /**< 强制走网宿的某一个ps链路 */
    QHVCEPROTO_FLV = 5, /**< FLV */
    QHVCEPROTO_MP4 = 6, /**< MP4 */
    QHVCEPROTO_M3U8 = 7 /**< M3U8 */
};

/// 发送数据时帧类型
enum QHVCEFrameTypeSend
{
    QHVCFRAME_AUDIO_SEND = 0, /**< 音频帧 */
    QHVCFRAME_VIDEO_SEND = 1 /**< 视频帧 */
};

/// 事件回调时的事件类型
enum QHVCEEvent
{
    QHVCEVENT_UNKNOWN = 0, /**< 非法事件 */
    QHVCEVENT_CONNECTED, /**< 成功连接到服务器(BinaryInfo*) */
    QHVCEVENT_CONNECT_FAILED, /**< 连接失败(无参数或int*) */
    QHVCEVENT_DISCONNECTED, /**< 连接断开(无参数) */
    QHVCEVENT_SN_ARRIVAL, /**< 调度成功(ServerAddrs*) */
    QHVCEVENT_SN_FAILED = 5, /**< 调度失败(int*) */
    QHVCEVENT_STREAM_CONNECT_FAIL, /**< 开流失败(无参数) */
    QHVCEVENT_STREAM_CONNECT, /**< 开流成功(无参数) */
    QHVCEVENT_STREAM_FINISH, /**< 主播推流结束(无参数) */
    QHVCEVENT_WRITELOCAL_FAILED, /**< 本地录制失败(无参数或WriteLocalEvent*) */
    QHVCEVENT_DROP_FRAME = 10, /**< 丢帧，内部事件 */
    QHVCEVENT_NEED_RESCHEDULING, /**< 需要重新调度(unsigned int*) */
    QHVCEVENT_NOT_HEALTHY_TRANS, /**< 推流效果较差（无参数）*/
    QHVCEVENT_RECONNECTING_START, /**< 开始重连（无参数） */
    QHVCEVENT_START_CHANNEL_OK, ///< 申请嘉宾成功(int*，嘉宾id)
    QHVCEVENT_STOP_CHANNEL_OK = 15, ///< 取消嘉宾成功(int*, 嘉宾id)
    QHVCEVENT_START_CHANNEL_FAILED, ///< 申请嘉宾失败(无参数)
    QHVCEVENT_START_PREVIEW, ///< 开始预览(int*，嘉宾id)
    QHVCEVENT_CHANNEL_ADD, ///< 增加一个嘉宾(UserOne*)
    QHVCEVENT_CHANNEL_DELETE, ///< 减少一个嘉宾(UserOne*)
    QHVCEVENT_CHANNEL_KICK = 20, ///< 踢掉一个嘉宾(int*, 嘉宾id)
    QHVCEVENT_SET_CHANNEL_AUDIO, ///< 关闭或打开一个嘉宾的声音(StatusToUpper*)
    QHVCEVENT_SET_CHANNEL_VIDEO, ///< 关闭或打开一个嘉宾的画面(StatusToUpper*)
    QHVCEVENT_SET_CHANNEL_NUM, ///< 设置嘉宾最大数量(int*)
    QHVCEVENT_APPLY_FOR_CHANNEL, ///< 申请一个嘉宾(char*，嘉宾名字)
    QHVCEVENT_USE_HWENCODER = 100, /**< 使用硬编码(无参数) */
    QHVCEVENT_AUTOADJUST_BITRATE, /**< 自适应码率（int*) */
    
    QHVCEVENT_DATA_RECEIVE = 999 /**< 内部事件 */
};

/// 调度返回信息结构
#define c_maxBackAddr (3)
struct QHVCServerAddrs
{
    enum QHVCEProto proto; ///< 协议类型
    const char *appKey; ///< app key
    const char *sn; ///< SN
    const char *encodeType; ///< 转码标识 h265或其他值
    const char *mainAddr; ///< 推流或看流地址
    const char *backAddr[c_maxBackAddr + 1]; ///< 备用地址
    const char *sn_alias;
    const char *publicMainAddr;
    
    int isRTC; ///< 是否rtc 1表示是 0表示不是
    int isMerge; ///< 是否合流 -1为无效值
    const char *isp; ///< 连麦服务商
    const char *appID; ///< 连麦的appID
    const char *token; ///< 连麦的token
    const char *aesKey; ///< 连麦的aesKey
};

/// 本地录制事件
enum QHVCWriteLocalEvent
{
    QHVCWRITE_SAMPLE_FAILED = 1, /**< 本地录制失败 */
    QHVCINVALID_FILE = 2, /**< 无效的存储位置 */
    QHVCILLEGAL_TS = 3 /**< 非法的时间戳 */
};

struct QHVCBinaryInfo
{
    const unsigned char *data; ///< 二进制数据
    size_t len; ///< 二进制数据长度
};

/// 帧类型
enum QHVCEFrameType
{
    QHVCFRAME_TYPE_UNKNOWN = -1, /**< 非法类型 */
    QHVCFRAME_TYPE_AUDIO, /**< OPUS */
    QHVCFRAME_TYPE_VIDEO_IDR, /**< IDR帧 */
    QHVCFRAME_TYPE_VIDEO_P, /**< P帧 */
    QHVCFRAME_TYPE_VIDEO_B, /**< B帧 */
    QHVCFRAME_TYPE_CONTROL, /**< 内部使用的帧类型 */
    QHVCFRAME_TYPE_AAC, /**< AAC */
    QHVCFRAME_TYPE_VIDEO_EXTRADATA,  /**<视频附加数据 （对于H264，这里是AVCDecoderConfigurationRecord, 对应ffmpeg里处理的extradata） */
    QHVCFRAME_TYPE_AUDIO_EXTRADATA   /**<音频附加数据  (对于AAC，这里是AudioSpecificConfig) */
};

/// 错误码
enum QHVCEErrCode
{
    QHVCTERROR_SUCCESS = 0, /**< 成功 */
    QHVCTERROR_INVALID_HANDLE    = -1, /**< 非法的实例句柄 */
    QHVCTERROR_INVALID_PARAM     = -2, /**< 非法参数 */
    QHVCTERROR_NOT_SUPPORT       = -3, /**< 不支持该操作 */
    QHVCTERROR_NOT_INITIALIZED   = -4, /**< 未初始化 */
    QHVCTERROR_BAD_CALL_SEQUENCE = -5, /**< 错误的调用顺序 */
    QHVCTERROR_SERVER = -6, ///< 与服务端通信错误
    QHVCTERROR_UNKNOWN           = -999, /**< 未知错误 */
};

/**
 * 事件回调函数原型
 * @param c 实例句柄
 * @param e 事件类型
 * @param param 事件参数
 * @param context 透传context
 */
typedef void (*qhvc_event_callback_t)(CORE_HANDLE c, enum QHVCEEvent e, const void *param, void *context);

/**
 * 数据回调函数原型
 * @param c 实例句柄
 * @param type 帧类型
 * @param buffer 数据
 * @param length 数据长度
 * @param context 上下文
 */
typedef void (*qhvc_frame_callback_t)(CORE_HANDLE c, enum QHVCEFrameType type, const char *buffer, int length, void *context);

/**
 * 编码后数据回调函数原型
 * @param c 实例句柄
 * @param type 帧类型
 * @param buffer 数据
 * @param length 数据长度
 * @param ts 时间戳
 * @param context 上下文
 */
typedef void (*qhvc_encoded_callback_t)(CORE_HANDLE c, enum QHVCEFrameType type, const unsigned char *buffer, unsigned int length,
                                   unsigned long long ts, void *context);

struct QHVCaddrinfo_dns
{
    int ai_flags;
    int ai_family;
    int ai_socktype;
    int ai_protocol;
    size_t ai_addrlen;
    struct sockaddr_storage ai_addr;
    struct addrinfo *ai_next;
};

/// 配置项
struct QHVCBaseSettings
{
    const char *sid; ///< 实例唯一ID
    
    qhvc_event_callback_t event_cb; ///< 事件回调函数
    void *event_context; ///< 事件回调函数上下文
    qhvc_frame_callback_t frame_cb; ///< 数据回调函数
    void *frame_context; ///< 数据回调函数上下文
    
    int retry_max; ///< 连接断开后重连尝试次数(私有)或尝试的时间(公有)
    int retry_maxTime; ///< 最长重试连接时间，默认180s
};

typedef struct
{
    unsigned int m_videoFrames; ///< 本次上报周期内视频帧数量
    unsigned int m_audioFrames; ///< 本次上报周期内音频帧数量
    unsigned long long m_bytes; ///< 本次上报周期内字节数
    unsigned int m_queueLen; ///< 队列长度
    unsigned long long m_queueBytes; ///< 队列内数据字节数
    unsigned int m_dropFrames; ///< 丢帧数
    unsigned int m_greater100ms; ///< 收到的数据时间戳间隔超过100ms的帧数
    unsigned int m_greater200ms; ///< 收到的数据时间戳间隔超过200ms的帧数
    unsigned int m_greater300ms; ///< 收到的数据时间戳间隔超过300ms的帧数
} QHVCStreamInfo;

typedef struct
{
    QHVCStreamInfo m_variantInfo;
    
    int m_connectionStatus; ///< 连接状态 0表示连接正常 1表示其他
    int m_proto; ///< 协议 0表示TCP 1表示UDP 2表示RTMP
    int m_eof; ///< 是否结束 0表示未结束 1表示结束
    int m_eofReason; ///< 结束原因 由业务设定
    int m_hardEncoderFlag; ///< 硬编标记 0表示软编 1表示硬编
    unsigned int m_retryCount; ///< 重连次数
    unsigned int m_switchCount; ///< 切换次数 暂时不使用
    unsigned int m_width; ///< 编码宽
    unsigned int m_height; ///< 编码高
    int m_trans264; ///< 转码264开关 0表示关闭 1表示开启 默认值为1
} QHVCStreamStatus;

// 一些可选参数
typedef struct
{
    char brand[20];
    char sysver[20];
    char area[20];//"cn"/"ap1"/"na1"/"eu1"
    char bsid[20];//godsees
} QHVCOptionalParams;

/**
 * 返回传输层版本号
 * @return 传输层版本号
 */
static __inline const char *qhvc_transport_version(void)
{
    return "4.2.0.21093001";
}
    
#ifdef __cplusplus
}
#endif

#endif /* QHVCTypes_h */
