//
//  QHVCEditTrackElement.h
//  QHVCEditKit
//
//  Created by liyue-g on 2019/8/13.
//  Copyright © 2019 liyue-g. All rights reserved.
//

#import "QHVCEditTrackClip.h"

#pragma mark - 元素对象基础方法

@interface QHVCEditElementFrame : NSObject
{
    NSInteger _timestamp;
    CVPixelBufferRef _pixelbuffer;
}

@end

@interface QHVCEditTrackElement : QHVCEditTrackClip

/**
 设置元素时长（单位：毫秒）

 @param duration 时长
 @return 返回值
 */
- (QHVCEditError)setDuration:(int64_t)duration;


/**
 获取元素时长（单位：毫秒）

 @return 时长
 */
- (int64_t)duration;

/***************************************************/
/******** 使用自定义元素时继承本类并重写以下方法 **********/
/***************************************************/

/**
 override: 初始化并打开元素数据源

 @return 返回值（成功0，失败<0）
 */
- (int)openElement;


/**
 override: 读取数据帧

 @param frame 输出数据帧
 @return 返回值（成功0，失败<0）
 */
- (int)readElementData:(QHVCEditElementFrame *)frame;


/**
 override: seek操作

 @param time seek至某个时间点
 @return 返回值（成功0，失败<0）
 */
- (int)seekElementToTime:(NSInteger)time;


/**
 override: 关闭元素数据源

 @return 返回值（成功0，失败<0）
 */
- (int)closeElement;

@end

#pragma mark - 画笔

@interface QHVCEditBrush : QHVCEditTrackElement
@property (nonatomic, assign) CGFloat lineWidth;    //画笔宽度
@property (nonatomic, assign) UIColor* lineColor;   //画笔颜色

@end

